module Diagrams

class Diagram
	def create_grid_from_string(string)
		lines = string.split("\n").map{|x| x.gsub(/\s*$/,'') }
		
		while lines.first && lines.first.strip.size == 0; lines.shift  end
		while lines.last && lines.last.strip.size == 0; lines.pop  end
		width = lines.map{|x| x.size}.max
		height = lines.size

		grid = Grid.new(width, height, 32)
		for y in 0..height-1
			for x in 0..width-1
				grid.set(x, y, lines[y][x])
			end
		end	
		grid
	end

	
	def detect_points
		[?+, ?*].each do |p|
			for_each_to_process(p) do |x,y|
				point = Point.new 
				point.pos = [x, y]
				point.char = p
				@to_process.set(x,y,false)
				@points.push point
				@occup.set(x,y, point)
			end
		end	
		
		@points.each do |p1|
			@points.each do |p2|
				if p1 != p2
					if p2.pos.x < p1.pos.x 
						p1.aligned[:t].push(p2) if p2.pos.y == p1.pos.y
					end
					if p2.pos.y < p1.pos.y 
						p1.aligned[:l].push(p2) if p2.pos.x == p1.pos.x
					end
				end
			end
		end
	end

	def detect_arrows
		[	[?|, /^\|*$/, [?^, ?A], [?v, ?V],  0, 1, :S],
			[?-, /^\-*$/, [?<], [?>],  1, 0, :E],
			[?\\,/^\\*$/, [], [], 1, 1, :SE],
			[?/, /^\/*$/, [], [], -1, 1, :SW] 
		].each do |char, reg, cap1, cap2, inc_x, inc_y, ty|
			for_each_to_process(char) do |x,y,e|
				find_arrow(x, y, reg, cap1, cap2, inc_x, inc_y)
			end
		end
	end
	
	
	def find_arrow(x, y, reg, cap1, cap2, inc_x, inc_y)
		inc = [inc_x, inc_y]
		rinc = [-inc_x, -inc_y]
		
		a = Arrow.new
		
		if cap1.include?(c = @text.get(x-inc_x, y-inc_y))
			a.cap1 = c
			a.start_pos = [x-inc_x, y-inc_y]
		else
			a.cap1 = nil
			a.start_pos = [x,y]
		end
		
		s = match(x,y,reg,inc_x,inc_y)
		len = s.size
		
		a.end_pos = [x+(len-1)*inc_x, y+(len-1)*inc_y]
		maybe_cap = [x+(len)*inc_x, y+(len)*inc_y]
		if cap2.include?(c=@text.get(*maybe_cap))
			len +=1
			a.end_pos = maybe_cap
			a.cap2 = c	
		else
			a.cap2 = nil
		end
		
		a.length = len
		
		before = [a.start_pos[0]-inc_x,a.start_pos[1]-inc_y]
		if ob = @occup.get(*before)
			impact = ob.link_direction(before, rinc)
			ob.links[impact] << a
			a.links[Reverse[impact]]<< ob
			a.start_link = ob
		else
			puts "Nothing found at #{before.inspect} : #{
			@text.get(*before)
			} while #{@text.get(*a.start_pos).chr}"
		end
		after = [a.end_pos[0]+inc_x,a.end_pos[1]+inc_y]
		if ob = @occup.get(*after)
			impact = ob.link_direction(after, inc)
			ob.links[impact] << a
			a.links[Reverse[impact]] << ob
			a.end_link = ob
		else
			puts "Nothing found at #{after.inspect} : #{
			@text.get(*after)
			} while #{@text.get(*a.end_pos).chr}"
		end
		
		iterate(a.start_pos,a.end_pos) do |x,y|
			@to_process.set(x,y,false)
			@occup.set(x,y,a)
		end
		
		@arrows.push a
		p a
	end
	
	def iterate(pos1,pos2)
		inc_x = pos2[0]-pos1[0]
		inc_y = pos2[1]-pos1[1]
		len = [inc_x.abs,inc_y.abs].max + 1
		if inc_x.abs > inc_y.abs
			len = inc_x.abs + 1
		else
			len = inc_y.abs + 1
		end
		
		inc_x  /= inc_x.abs if inc_x != 0
		inc_y  /= inc_y.abs if inc_y != 0
		
#		raise "" if inc_x 
		for i in 0..(len-1)
			x, y = pos1[0]+i*inc_x,pos1[1]+i*inc_y
			yield x,y
		end
	end
	
	def detect_boxes
		for_each_to_process(?+) do |x,y,e|
			find_box(x,y,/^\+[\-\+]+\+$/, /^\+[\|\+]+\+$/)
		end
		for_each_to_process(?/) do |x,y,e|
			find_box(x,y,/^\/[\-\+]+\\$/, /^\/[\|\+]+\\$/)
		end
		
		# find alignments
		@boxes.each do |b1|
			@boxes.each do |b2|
				if b1 != b2
					if (b2.y < b1.y)
						b1.aligned[:l].push(b2) if (b1.x == b2.x)
						b1.aligned[:r].push(b2) if (b1.x+b1.width == b2.x+b2.width)
					end
					if (b2.x < b1.x)
						b1.aligned[:t].push(b2) if (b1.y == b2.y)
						b1.aligned[:b].push(b2) if (b1.y+b1.height == b2.y+b2.height)
					end
				end
			end
		end
	end
	
	def for_each_to_process(char) 
		@text.each do |x,y,e|
			if e === char && @to_process.get(x,y)
				yield x, y, e
			end
		end
	end
	
	def find_box(x,y,hmatch,vmatch)
		horiz = match_horiz(x,y,hmatch)
		vert  = match_vert(x,y,vmatch)
		
		return if not horiz or not vert
		width, height = horiz.size, vert.size
		if (width > 2) && (height > 2)
			# mark the cell as processed
			@to_process.set_area(x, y, horiz.size, vert.size, false)
			box = Box.new(x,y,width,height)
			box.content = @text.read_area(x+1,y+1,width-2,height-2)
			box.content.gsub!(/\n/,' ')			
			# mark the area as occupied by the box
			@occup.set_area(x, y, horiz.size, vert.size, box)
			@boxes.push box
		end
	end

	def match(x,y,reg,inc_x,inc_y)
		s = ""
		best = nil
		while x<@text.width && y < @text.height
			s << @text.get(x,y).chr
			break if not @to_process.get(x,y)
			if s =~ reg
				best = s.dup 
			end
			x += inc_x; y += inc_y
		end
		best
	end
	
	def match_horiz(x,y,reg)
		match(x,y,reg,1,0)
	end
	
	def match_vert(x,y,reg)
		match(x,y,reg,0,1)
	end
	
	
end


end # moudle Diagram