class Array
	def x() self[0] end
	def y() self[1] end
end

module Diagrams	
	
class Spring
	attr_accessor :min,:shrink,:pref,:stretch,:max
	attr_accessor :weight  #not used yet		
	attr_accessor :imposed
	
	def initialize(min,shrink,pref,stretch,max)
		@min,@shrink,@pref,@stretch,@max = 
		 min, shrink, pref, stretch, max
	end
	
	def Spring.series(s1,s2)
		Spring.new( 
			s1.min+s2.min, 
			s1.shrink+s2.shrink,
			s1.pref+s2.pref,
			s1.stretch+s2.stretch,
			s1.max+s2.max
			)
	end
	
	def Spring.parallel(s1,s2)
		pref = (s1.pref+s2.pref)*0.5
		pref = [s1.min, pref].max
		pref = [s1.max, pref].min
		pref = [s2.min, pref].max
		pref = [s2.max, pref].min
		
		Spring.new( 
			[s1.min,s2.min].max, 
			[s1.shrink,s2.shrink].min,
			pref,
			[s1.stretch,s2.stretch].min,
			[s1.max,s2.max].min
			)
	end
	
	
	def inspect
		if @min == @max
			"Fixed(#{@min})"
		else
			"S[%.1f(%.1f)%.1f(%.1f)%.1f]=%.1f" %
			[ @min, @shrink, @pref, @stretch, @max, @imposed || -1]
		end
	end
end

Reverse = {:N=>:S,:S=>:N,:W=>:E,:E=>:W}

class Sized
	attr_accessor :hs, :vs # horizontal, vertical spring
	attr_accessor :links #hash :N,:S,:W,:E
	def initialize
		@hs = Spring.new(1,0,1,0,1)
		@vs = Spring.new(1,0,1,0,1)
		@links = {:N=>[],:E=>[],:W=>[],:S=>[]}
	end
end

class Arrow < Sized
	attr_accessor :type # :S, :NW, :SE, :E
	attr_accessor :cap1,:cap2 # either nil or the cap
	attr_accessor :start_pos, :end_pos
	attr_accessor :start_link, :end_link
	attr_accessor :length
	attr_accessor :object_start, :object_end


	# variables for start and end
	attr_accessor :v_start,:v_end
	
	def is_vert?
		start_pos.x == end_pos.x
	end
	
end


class Box < Sized
	attr_accessor :x,:y,:width,:height
	attr_accessor :content

	attr_accessor :aligned

	# variables for top-left and bottom-right
	attr_accessor :v_tl,:v_br

	
	def initialize(x,y,width,height)
		super()
		@x,@y,@width,@height = x,y,width,height
		@aligned = {:l=>[],:t=>[],:b=>[],:r=>[]}
	end

	# pos: position touching this
	# from: other
	def link_direction(pos, inc)
		if pos.x == @x then return :W end
		if pos.x == @x+@width-1 then return :E end
		if pos.y == @y then return :N end
		if pos.y == @y+@height-1 then return :S end
		raise "Strange #{pos.inspect}: "+self.inspect
	end
end

class Point < Sized
	attr_accessor :pos
	attr_accessor :char

	attr_accessor :aligned
	
	# variables for point position
	attr_accessor :v_p
	
	def initialize
		super()
		@aligned = {:l=>[],:t=>[]}
	end
	
	def link_direction(pos, inc)
		if inc.x > 0 then return :W end
		if inc.x < 0 then return :E end
		if inc.y > 0 then return :N end
		if inc.y < 0 then return :S end
		raise "Strange #{inc.inspect}: "+self.inspect
	end
end


class Diagram
	attr_accessor :boxes, :points, :arrows
	
	attr_accessor :text, :occup, :to_process
	
	def initialize(s)
		@text = create_grid_from_string(s)
		@to_process = Grid.new(@text.width, @text.height, false)
		@occup = Grid.new(@text.width, @text.height, nil)
		@boxes = []
		@points = []
		@arrows = []
		# mark cells to process
		@text.each do |x,y,e| 
			@to_process.set(x,y,true) if e != 32
		end
		@width, @height = @text.width,@text.height

	#	p @to_process
		
		puts @text.inspect2(@to_process)
		
		detect_boxes
		detect_points
		detect_arrows

		puts @text.inspect2(@to_process)
	end
end


end